# ----------------------------------------------------------------
#  DataManager module:
#    Classes
#        DataBaseInterface: Abstract database interface
#        DataG: Interace to Gadfly database
#        DataB: Interace to directory/file database
#        QueueManager: Manages low & hi priority Queues
#        ListManager: lo overhead QM
#        DataInterface: Manages listManager & DataBaseInterface
# ----------------------------------------------------------------
import Queue
import exceptions
import time
import sys
import marshal
import random
import os
import threading
from Util import *

# ----------------------------------------------------------------
#  DataBaseInterface: Abstract base class.
#
# ----------------------------------------------------------------
class DataBaseInterface:
    def __init__(self):
        raise NotImplementedError, 'DataBaseInterface.__init__()'
    # ----------------------------------------------------------------
    #  to_string: returns string representation of
    #     database's contents
    # ----------------------------------------------------------------
    def to_string(self):
        raise NotImplementedError, 'DataBaseInterface.to_string()'
    # ----------------------------------------------------------------
    #  isEmpty: return 1 if empty, 0 otherwise
    #
    # ----------------------------------------------------------------
    def isEmpty(self):
        raise NotImplementedError, 'DataBaseInterface.isEmpty()'
    # ----------------------------------------------------------------
    #  insert: creates & returns a new handle associated
    #     with a new entry into database.
    # ----------------------------------------------------------------
    def insert(self, handle, proto, priority, url_src, url_dest, credPath=''):
        raise NotImplementedError, 'DataBaseInterface.insert()'
    # ----------------------------------------------------------------
    #  clear_entry: removes all information associated with a
    #     handle.
    # ----------------------------------------------------------------
    def clear_entry(self, handle):
        raise NotImplementedError, 'DataBaseInterface.clear_entry()'
    # ----------------------------------------------------------------
    #  access methods:
    # ----------------------------------------------------------------
    def get_credPath(self, handle):
        raise NotImplementedError, 'DataBaseInterface.get_credPath()'
    def set_credPath(self, handle):
        raise NotImplementedError, 'DataBaseInterface.set_credPath()'
    def get_marker(self, handle):
        raise NotImplementedError, 'DataBaseInterface.get_marker()'
    def set_marker(self, handle, marker):
        raise NotImplementedError, 'DataBaseInterface.set_marker()'
    def get_priority(self, handle):
        raise NotImplementedError,  'DataBaseInterface.get_priority()'
    def set_priority(self, handle, priority):
        raise NotImplementedError, 'DataBaseInterface.set_priority()'
    def get_status(self, handle):
        raise NotImplementedError, 'DataBaseInterface.get_status()'
    def set_status(self, handle, status):
        raise NotImplementedError, 'DataBaseInterface.set_status()'
    # ----------------------------------------------------------------
    #  get_tuple_of_handles_of_status:
    #     return:  tuple('handle', ..)
    # ----------------------------------------------------------------
    def get_tuple_of_handles_of_status(self,status):
        raise NotImplementedError,\
              'DataBaseInterface.get_tuple_of_handles_of_status()'
    # ----------------------------------------------------------------
    #  get_transfer_info: Get transfer information associated with
    #    a handle.
    #    return:  tuple('source url', 'destination url', \
    #                   'marker', 'credential path')
    # ----------------------------------------------------------------
    def get_transfer_info(self, handle):
        raise NotImplementedError, 'DataBaseInterface.get_transfer_info()'
    # ----------------------------------------------------------------
    #  get_record: Get a handle's record from database.
    #    return:  tuple('prototype', 'source url', 'destination url', \
    #                   'marker', 'status', 'priority')
    # ----------------------------------------------------------------
    def get_record(self, handle):
        raise NotImplementedError, 'DataBaseInterface.get_record()'

# ----------------------------------------------------------------
#  DataG: interface to gadfly database
#
# ----------------------------------------------------------------
class DataG(DataBaseInterface):
    def __init__(self, db_name, db_dir, db_table):
        import gadfly
        try:
            self._conn = gadfly.gadfly(db_name, db_dir)
        except IOError, ex:
            Log_event('Creating gadfly database')
            try:
                os.mkdir(db_dir)
            except OSError,ex:
                Log_debug(("Directory exists: %s" %(db_dir)), 5)
            else:
                Log_debug(("Created directory: %s" %(db_dir)), 5)
            self._conn = gadfly.gadfly()
            self._conn.startup(db_name, db_dir)
        self._cur = self._conn.cursor()
        try:
            self._cur.execute(("create table ") + db_table + \
                               (" (h varchar, p varchar, s varchar, ") + \
                               ("proto varchar, src varchar, dst varchar,") + \
                               ("m varchar, c varchar)"))
        except NameError,ex:
            Log_debug(('gadfly database table %s exists' %(db_table)), 5)
        except IOError,msg:
            Log_exception(ex)
            sys.exit(-1)
        else:
            Log_debug(("New Table created: %s" %(db_table)), 5)
        self._db_table = db_table
        self._select_jobs = "select * from " + db_table
        self._insert_job = "insert into " + db_table + \
                           ("(h, proto, p, s, src, dst, m, c) " +
                            "values (?, ?, ?, ?, ?, ?, ?, ?)")
        self._select_job = "select * from " + db_table + " where h=?"
        self._delete_job = "delete from " + db_table + " where h=?"
        self._select_h = "select h from " + db_table + " where h=?"
        self._select_p = "select p from " + db_table + " where h=?"
        self._select_s = "select s from " + db_table + " where h=?"
        self._select_m = "select dst from " + db_table + " where h=?"
        self._select_m = "select src from " + db_table + " where h=?"
        self._select_c = "select c from " + db_table + " where h=?"
        self._select_proto = "select proto from " + db_table + " where h=?"
	self._update_c = "update " + db_table + " set c=? where h=?"
        self._update_p = "update " + db_table + " set p=? where h=?"
        self._update_s = "update " + db_table + " set s=? where h=?"
        self._update_m = "update " + db_table + " set m=? where h=?"
        self._get_handles_status = 'select h from ' + db_table + ' where s=?'
        # Keys into Gadfly results
        self._h_key = 'H'
        self._s_key = 'S'
        self._p_key = 'P'
        self._c_key = 'C'
        self._src_key = 'SRC'
        self._dst_key = 'DST'
        self._m_key = 'M'
        self._proto_key = 'PROTO'
        return
    def __str__(self):
        self._cur.execute(self._select_jobs)
        tmp = ''
        up  = urlparser()
        cnt = len(self._cur.results[0].tuples)
        for i in self._cur.results[0].tuples:
            p   = i.dict.get(self._proto_key)
            tmp = (tmp + i.dict.get(self._h_key) + 
                  '#' + up.urlunparse(p, i.dict.get(self._src_key)) + 
                  '#' + up.urlunparse(p, i.dict.get(self._dst_key)) +
                  '#' + str(i.dict.get(self._s_key)) +
                  '#' + str(i.dict.get(self._p_key)) +
                  '#' + i.dict.get(self._m_key) + '#0#\n')

        return ('#database#' + str(cnt) + '\n' + tmp)
    def __check_for_entry(self):
        try:
            items = len(self._cur.results[0].tuples) #list of dictionaries
        except AttributeError, ex:
            Log_exception(ex)
            raise RuntimeError, 'called check_for_entry cursor update/?'
        else:
            if  items > 1:
                raise RuntimeError, 'Multiple entries for handle'
            if  not items:
                raise ValueError, 'No entry for handle'
        return
    def __handle_submitted(self, handle):
        self._cur.execute(self._select_h, (handle,))
        result_list = self._cur.results[0].tuples #list of dictionaries
        items = len(result_list)
        if items == 1:
            raise ValueError, 'handle already submitted'
        elif items > 1:
            raise RuntimeError, 'Multiple entries for handle'
        return
    def isEmpty(self):
        self._cur.execute(self._select_jobs)
        result_list = self._cur.results[0].tuples
        return not len(result_list)
    def to_string(self):
        return self.__str__()
    
    # ----------------------------------------------------------------
    #  insert:
    # ----------------------------------------------------------------
    def insert(self, handle, proto, priority, url_src, url_dest, credPath=''):
        self.__handle_submitted(handle)
        self._cur.execute(self._insert_job, (handle, proto, priority, 0,
                                             url_src, url_dest, '', credPath))
        self._conn.commit()
        return handle
    # ----------------------------------------------------------------
    #  clear_entry:
    # ----------------------------------------------------------------
    def clear_entry(self, handle):
        self._cur.execute(self._delete_job, (handle,))
        self._conn.commit()
        return
    # ----------------------------------------------------------------
    #  access methods:
    # ----------------------------------------------------------------
    def get_credPath(self, handle):
        self._cur.execute(self._select_c, (handle,))
        self.__check_for_entry()
        c_dict = self._cur.results[0].tuples[0].dict
        c = c_dict.get(self._c_key, -1)
        if c == -1:
            raise ValueError, 'No credential for handle'
        return c
    def set_credPath(self, handle, credPath):
        self._cur.execute(self._update_c, (credPath, handle))
        self._conn.commit()
        return
    def get_marker(self, handle):
        self._cur.execute(self._select_m, (handle,))
        self.__check_for_entry()
        m_dict = self._cur.results[0].tuples[0].dict
        m = m_dict.get(self._m_key, -1)
        if m == -1:
            raise ValueError, 'No marker for handle'
        return m
    def set_marker(self, handle, marker):
        self._cur.execute(self._update_m, (marker, handle))
        self._conn.commit()
        return
    def get_priority(self, handle):
        self._cur.execute(self._select_p, (handle,))
        self.__check_for_entry()
        p_dict = self._cur.results[0].tuples[0].dict
        p = p_dict.get(self._p_key, -1)
        if p == -1:
            raise ValueError, 'No priority for handle'
        return p
    def set_priority(self, handle, priority):
        self._cur.execute(self._update_p, (priority, handle))
        self._conn.commit()
        return
    def get_status(self, handle):
        self._cur.execute(self._select_s, (handle,))
        self.__check_for_entry()
        s_dict = self._cur.results[0].tuples[0].dict
        s = s_dict.get(self._s_key, -1)
        if s == -1:
            raise ValueError, 'No status for handle'
        return s
    def set_status(self, handle, status):
        self._cur.execute(self._update_s, (status, handle))
        self._conn.commit()
        return
    # ----------------------------------------------------------------
    #  get_tuple_of_handles_of_status:
    # ----------------------------------------------------------------
    def get_tuple_of_handles_of_status(self,status):
        self._cur.execute(self._get_handles_status, (status,))
        h_list = []
        for i in self._cur.results[0].tuples:
            h_list.append(i.dict.get(self._h_key))
        return tuple(h_list)
    # ----------------------------------------------------------------
    #  get_transfer_info:
    # ----------------------------------------------------------------
    def get_transfer_info(self, handle):
        self._cur.execute(self._select_job, (handle,)) 
        self.__check_for_entry()
        r_dict = self._cur.results[0].tuples[0].dict
        #s = r_dict.get(self._s_key, -1)
        #p = r_dict.get(self._p_key, -1)
        proto = r_dict.get(self._proto_key, '')       
        src = r_dict.get(self._src_key, '')
        dst = r_dict.get(self._dst_key, '')
        m = r_dict.get(self._m_key, '')
        c = r_dict.get(self._c_key, '')
        up = urlparser()
        url_src = up.urlunparse(proto, src)
        url_dst = up.urlunparse(proto, dst)
        t = (url_src, url_dst, m, c)
        return t
    # ----------------------------------------------------------------
    #  get_record:
    # ----------------------------------------------------------------
    def get_record(self, handle):
        self._cur.execute(self._select_job, (handle,))
        self.__check_for_entry()
        r_dict = self._cur.results[0].tuples[0].dict
        s = r_dict.get(self._s_key, -1)
        p = r_dict.get(self._p_key, -1)
        m = r_dict.get(self._m_key, '')
        src = r_dict.get(self._src_key, '')
        dst = r_dict.get(self._dst_key, '')
        proto = r_dict.get(self._proto_key, '')
        up = urlparser()
        url_src = up.urlunparse(proto, src)
        url_dst = up.urlunparse(proto, dst)
        t = (url_src, url_dst, m, s, p)
        return t

# ----------------------------------------------------------------
#  DataB: directory implementation of database.
#
#
# ----------------------------------------------------------------
class DataB(DataBaseInterface):
    def __init__(self, db_file, db_dir, db_table):
        self._handleList = []
        self._protoDir = {}
        self._statusDir = {}
        self._priorityDir = {}
        self._sendDir = {}
        self._recvDir = {}
        self._restartDir = {}
        self._credDir = {}
        if db_dir:
            self._file = db_dir + '/' + db_file
            try:
                os.mkdir(db_dir)
            except OSError,ex:
                Log_debug(("Directory exists: %s" %(db_dir)), 4)
            else:
                Log_debug(("Created directory: %s" %(db_dir)), 4)
        else:
            self._file = db_file
        try:
            print "FILE: ", self._file
            f = open(self._file, 'r')
        except IOError, ex:
            Log_exception(ex)
        else:
            try:
                t = marshal.load(f)
            except EOFError, ex:
                Log_exception(ex)
            except ValueError, ex:
                Log_exception(ex)
            except TypeError, ex:
                Log_exception(ex)
            else:
                if len(t) != 8:
                    f.close()
                    raise RuntimeError, \
                          'DataB passed a Corrupted DataBase File'
                else:
                    (self._handleList, self._protoDir, self._statusDir, 
                     self._priorityDir, self._sendDir, self._recvDir, 
                     self._restartDir,  self._credDir) = t
                    f.close()
        return
    def __str__(self):
        tmp = ''
        cnt = len(self._handleList)
        up  = urlparser()
        for handle in self._handleList:
             p   = self._protoDir.get(handle)
             src = up.urlunparse(p, self._sendDir.get(handle))
             dst = up.urlunparse(p, self._recvDir.get(handle))
             tmp = (tmp + handle + '#' + src + '#' + dst
                    + '#' + str(self._statusDir.get(handle))
                    + '#' + str(self._priorityDir.get(handle))
                    + '#' + self._restartDir.get(handle) + '#0#\n')
        return ('#database#' + str(cnt) + '\n' + tmp)
    def __write_to_file(self):
        try:     
            f = open(self._file, 'w+')
        except IOError, ex:
            Log_exception(ex)
        else:
            t = (self._handleList, self._protoDir, self._statusDir,\
                 self._priorityDir,self._sendDir, self._recvDir,\
                 self._restartDir, self._credDir)
            marshal.dump(t, f)
            f.close()
        return
    def __handle_submitted(self, handle):
        s = self._statusDir.get(handle, -1)
        if s >= 0:
            raise ValueError, 'handle already submitted'
        return
    def isEmpty(self):
        isE = len((self._statusDir).values()) + \
              len(self._priorityDir.values()) + \
              len(self._sendDir.values()) + \
              len(self._recvDir.values()) + \
              len(self._restartDir.values())
        return ( isE == 0 )
    def to_string(self):
        return self.__str__()

    # ----------------------------------------------------------------
    #  insert:
    # ----------------------------------------------------------------
    def insert(self, handle, proto, priority, url_src, url_dest, credPath=''):
        self.__handle_submitted(handle)
        self._handleList.append(handle)
        self._protoDir.update({handle:proto})
        self._priorityDir.update({handle:priority})
        self._sendDir.update({handle:url_src})
        self._recvDir.update({handle:url_dest})
        self._credDir.update({handle:credPath})
        self._statusDir.update({handle:0})
        self._restartDir.update({handle:''})
        self.__write_to_file()
        return handle
    # ----------------------------------------------------------------
    #  clear_entry:
    # ----------------------------------------------------------------
    def clear_entry(self, handle):
        self._handleList.remove(handle)
        dirList = [self._protoDir, self._priorityDir, self._sendDir, 
                   self._recvDir, self._credDir, self._statusDir, 
                   self._restartDir]
        try:
             for d in dirList: 
                  del d[handle]
        except KeyError, ex:
             raise ValueError, 'clear_entry incomplete entry'

        self.__write_to_file()
        return
    
    # ----------------------------------------------------------------
    #  access methods:
    # ----------------------------------------------------------------
    def get_credPath(self, handle):
        c = self._credDir.get(handle, -1)
        if c == -1:
            raise ValueError, 'No credential Path for handle'
        return c
    def get_marker(self, handle):
        m = self._restartDir.get(handle, -1)
        if m == -1:
            raise ValueError, 'No restartMarker for handle'
        return m
    def set_marker(self, handle, marker):
        self._restartDir.update({handle:marker})
        self.__write_to_file()
        return
    def get_priority(self, handle):
        p = self._priorityDir.get(handle, -1)
        if p == -1:
            raise  ValueError, 'No priority for handle'
        return p
    def set_priority(self, handle, priority):
        self._priorityDir.update({handle:priority})
        self.__write_to_file()
    def get_status(self, handle):
        s = self._statusDir.get(handle, -1)
        if s == -1:
            raise ValueError, 'No status for handle'
        return s
    def set_status(self, handle, status):
        self._statusDir.update({handle:status})
        self.__write_to_file()
        return
    # ----------------------------------------------------------------
    #  get_tuple_of_handles_of_status:
    # ----------------------------------------------------------------
    def get_tuple_of_handles_of_status(self,status):
        l = []
        for h in self._statusDir.keys():
            s = self._statusDir.get(h, -1)
            if status == s:
                l.append(h)
        return tuple(l)
    # ----------------------------------------------------------------
    #  get_transfer_info:
    # ----------------------------------------------------------------
    def get_transfer_info(self, handle):
        #s = self.get_status(handle)
        #p = self.get_priority(handle)
        proto = self._protoDir.get(handle, '')
        src  = self._sendDir.get(handle, '')
        dst = self._recvDir.get(handle, '')
        m = self.get_marker(handle)
        c = self.get_credPath(handle)
        up = urlparser()
        url_src = up.urlunparse(proto, src)
        url_dst = up.urlunparse(proto, dst)
        t = (url_src, url_dst, m, c)
        return t
    # ----------------------------------------------------------------
    #  get_record:
    # ----------------------------------------------------------------
    def get_record(self, handle):
        print "def get_record(self, handle)"
        s = self.get_status(handle)
        p = self.get_priority(handle)
        m = self.get_marker(handle)
        proto = self._protoDir.get(handle, '')
        src  = self._sendDir.get(handle, '')
        dst = self._recvDir.get(handle, '')
        up = urlparser()
        url_src = up.urlunparse(proto, src)
        url_dst = up.urlunparse(proto, dst)
        t = (url_src, url_dst, m, s, p)
        return t
    
# ----------------------------------------------------------------
#  ListManager: Manages 2 lists, one lo priority and one hi priority
#
#
# ----------------------------------------------------------------
class ListManager:
    def __init__(self):
        self._hiList = []
        self._loList = []
    def __str__(self):
        lo_str = '#loqueue#' + str(len(self._loList)) + '\n'
        hi_str = '#hiqueue#' + str(len(self._hiList)) + '\n'
        for l in self._loList:
            lo_str += (l + '\n')
        for h in self._hiList:
            hi_str += (h + '\n')
        return (lo_str + hi_str)
    def to_string(self):
        return self.__str__()
    def isEmpty(self):
        return ((len(self._hiList) + len(self._loList)) == 0)
    # ----------------------------------------------------------------
    #  put: put 'handle' onto appropriate queue.
    # ----------------------------------------------------------------
    def put(self, handle, priority):
        if priority == LO_PRIORITY:
            self._loList.append(handle)
        elif priority == HI_PRIORITY:
            self._hiList.append(handle)
        else:
            raise ValueError, 'invalid priority'
        return
    # ----------------------------------------------------------------
    #  get_next: retrieves next handle
    # ----------------------------------------------------------------
    def get_next(self):
        h = ''
        if len(self._hiList): 
            h = self._hiList[0]
            self._hiList = self._hiList[1:]
        elif (self._loList):
            h = self._loList[0]
            self._loList = self._loList[1:]
        return h
    # ----------------------------------------------------------------
    #  move: moves specified handle to new_priority queue if it is
    #      in a different queue. Throws RuntimeError if priority is
    #      invalid or if move didn't happen.
    # ----------------------------------------------------------------
    def move(self, handle, new_priority):
        if new_priority == HI_PRIORITY: #0->1
            try:
                self._loList.remove(handle)
            except ValueError, ex:
                raise RuntimeError, 'move failed, item not in lo list'
            self._hiList.append(handle)
        elif new_priority == LO_PRIORITY: #1->0
            try:
                self._hiList.remove(handle)
            except ValueError, ex:
                raise RuntimeError, 'move failed, item not in hi list'
            self._loList.append(handle)
        else:
            raise RuntimeError, 'move failed, new_priority invalid'
        return
    # ----------------------------------------------------------------
    #  remove: remove handle if it exists in either queue.  Throws
    #      RuntimeError if handle is invalid.
    #
    # ----------------------------------------------------------------
    def remove(self, handle):
        try:
            self._hiList.remove(handle)
        except ValueError, ex:
            try:
                self._loList.remove(handle)
            except ValueError, ex:
                raise RuntimeError, 'remove failed, no such handle'
        return

# ----------------------------------------------------------------
#  QueueManager: Manages 2 queues, one lo priority and one hi priority
#
#
# ----------------------------------------------------------------
class QueueManager:
    def __init__(self):
        self._hiQueue = Queue.Queue()
        self._loQueue = Queue.Queue()
    def __str__(self):
        lo_str = '#loqueue#' + str(self._loQueue.qsize()) + '\n'
        hi_str = '#hiqueue#' + str(self._hiQueue.qsize()) + '\n'
        lo_list = self._loQueue.queue
        hi_list = self._hiQueue.queue
        for h in lo_list:
            lo_str = lo_str +  h + '\n'
        for h in hi_list:
            hi_str = hi_str + h + '\n'
        return (lo_str + hi_str)
    def to_string(self):
        return self.__str__()
    def isEmpty(self):
        return (self._hiQueue.empty() and self._loQueue.empty())
    # ----------------------------------------------------------------
    #  put: put 'handle' onto appropriate queue.
    # ----------------------------------------------------------------
    def put(self, handle, priority):
        if priority == LO_PRIORITY:
            self._loQueue.put(handle)
        elif priority == HI_PRIORITY:
            self._hiQueue.put(handle)
        else:
            raise ValueError, 'invalid priority'
        return
    # ----------------------------------------------------------------
    #  get_next: retrieves next handle
    # ----------------------------------------------------------------
    def get_next(self):
        h = ''
        if self._hiQueue.qsize():
            h = self._hiQueue.get()
        elif self._loQueue.qsize():
            h = self._loQueue.get() 
        return h
    # ----------------------------------------------------------------
    #  move: moves specified handle to new_priority queue if it is
    #      in a different queue. Throws RuntimeError if priority is
    #      invalid or if move didn't happen.
    # ----------------------------------------------------------------
    def move(self, handle, new_priority):
        tempQ = Queue.Queue()
        queue = None
        target = None
        ret = 0
        if new_priority == HI_PRIORITY: #0->1
            queue = self._loQueue
            target = self._hiQueue
        elif new_priority == LO_PRIORITY: #1->0
            queue = self._hiQueue
            target = self._loQueue
        else:
            raise RuntimeError, 'move failed, new_priority invalid'
        while queue.qsize():
            temp = queue.get()
            if temp != handle:
                tempQ.put(temp)
            else:
                target.put(handle)
                ret = 1
        if new_priority: # referenced to _loQueue/_hiQueue
            self._loQueue = tempQ
        else:
            self._hiQueue = tempQ
        if not ret:
            raise RuntimeError, 'move failed, not in move from queue'
        return
    # ----------------------------------------------------------------
    #  remove: remove handle if it exists in either queue.  Throws
    #      RuntimeError if handle is invalid.
    #
    # ----------------------------------------------------------------
    def remove(self, handle, priority=0):
        ret = 0
        for i in range(2):
            tempQ = Queue.Queue()
            if priority == 0:
                while self._loQueue.qsize():
                    temp = self._loQueue.get(handle)
                    if temp == handle:
                        ret = 1
                    else:
                        tempQ.put(temp)
                self._loQueue = tempQ
            else:
                while self._hiQueue.qsize():
                    temp = self._hiQueue.get()
                    if temp == handle:
                        ret = 1
                    else:
                        tempQ.put(temp)
                self._hiQueue = tempQ
            if ret:break
            del tempQ
            priority = not priority
        if not ret:
            raise RuntimeError, 'No such handle'
        return
    
# ----------------------------------------------------------------
#  DataInterface:  Manages state transitions of jobs and the
#     Database and queueManager objects.
#     - singleton class
# ----------------------------------------------------------------
def get_DataInterface():
    if not DataInterface.singleton:
        raise RuntimeError, \
              'singleton class, call constructor to initialize'
    return DataInterface.singleton

class DataInterface:
    singleton = None
    def __init__(self, db, db_name, db_file, db_table):
        if DataInterface.singleton:
            raise RuntimeError, \
                  'singleton class instantiated, use get_DataInterface method'
        DataInterface.singleton = self
        self._condV    = threading.Condition(threading.Lock())
        self._dataBase = (eval(db))(db_name, db_file, db_table)
        self._queueM  = ListManager()
        self.__recommit_jobs()
        return
    def __str__(self):
        self._condV.acquire(1)
        try:
            msg = (self._dataBase.__str__()+ self._queueM.__str__())
        finally:
            self._condV.release()
        return msg
            
    def __check_status(self, status):
        if type(1) != type(status):
            raise TypeError, 'status must be an int'
        if (status > MAX_STATUS) or (status < 0):
            raise ValueError, 'invalid status'
        return
    def __check_priority(self, priority):
        if type(1) != type(priority):
            raise TypeError, 'priority must be an int'
        if not (priority == LO_PRIORITY or priority == HI_PRIORITY):
            raise ValueError, 'invalid priority'
        return
    def __recommit_jobs(self):
        handle_tuple = ()
        for state in ACTIVE_STATES:
            handle_tuple += \
                         self._dataBase.get_tuple_of_handles_of_status(state)
        for h in handle_tuple:
            self._dataBase.set_status(h, COMT)
            p = self._dataBase.get_priority(h)
            try:
                self._queueM.put(h, p)
            except ValueError, ex:
                Log_exception(ex)
                Log_debug(4, self._dataBase.get_record(h))
                self._dataBase.set_status(h, FAIL)
        return
    def to_string(self):
        return self.__str__()
    def job_ready(self):
        return not self._queueM.isEmpty()
    # ----------------------------------------------------------------
    #  accessors:
    #
    # ----------------------------------------------------------------
    def get_status(self, handle):
        s = self._dataBase.get_status(handle)
        return s
    def get_credPath(self, handle):
        c = self._dataBase.get_credPath(handle)
        return c
    def get_marker(self, handle):
        m = self._dataBase.get_marker(handle)
        return m
    def set_marker(self, handle, marker):
        self._dataBase.set_marker(handle, marker)
        return
    # ----------------------------------------------------------------
    #  submit: Insert job into database, if successful database
    #     hands back a handle. Throw ValueError or TypeError if
    #     priority is invalid.  If handle is supplied, attempt
    #     insert once, else supply a unique one.
    # ----------------------------------------------------------------
    def submit_job(self, url_src, url_dst, priority, credPath='', handle=None):
        self.__check_priority(priority)
        random.seed()
        up = urlparser()
        tmp = up.urlparse(url_src)
        src = tmp[1] + tmp[2]
        tmp = up.urlparse(url_dst)
        dst = tmp[1] + tmp[2]
        while not handle:
            h = str(random.random())
            try:
                h = self._dataBase.insert(h[2:], tmp[0], int(priority), src, dst,
                                          credPath)
            except ValueError, ex:
                Log_exception(ex)
                time.sleep(0.1)
            else:
                break
        else:
            h = self._dataBase.insert(handle, tmp[0], int(priority), src, dst,
                                      credPath)
        return h
    # ----------------------------------------------------------------
    #  commit_job: If status of handle from database is listed in
    #     COMMIT_STATES, then change it's status in database to
    #     COMT and put the handle in the ListManager.
    #     *ValueError - User provided bad/inappropriate handle
    # ----------------------------------------------------------------
    def commit_job(self, handle):
        self._condV.acquire(1)
        try:
            s = self._dataBase.get_status(handle)
            for state in COMMIT_STATES:
                if s == state:
                    p = self._dataBase.get_priority(handle)
                    self._queueM.put(handle, p)
                    self._dataBase.set_status(handle, COMT)
                    break
            else:
                raise StatusError('handle is not in committable state',
                                  g_status_dict.get(s))
        finally:
            self._condV.release()
        return handle
    # ----------------------------------------------------------------
    #  get_next_job:  returns tuple with transfer information
    #     will throw ValueError if job is not in active state.
    #     t = tuple('handle', 'prototype', 'source url', 'destination url', \
    #             'marker', 'credential path')
    # ----------------------------------------------------------------
    def get_next_job(self):
        l = []
        self._condV.acquire(1)
        try:
            h = self._queueM.get_next()
            if h:
                p = self._dataBase.get_priority(h)
                s = self._dataBase.get_status(h)
                if s != COMT or s == -1 or p < 0:
                    self._dataBase.set_status(h, FAIL)
                    raise RuntimeError,\
                          'queue/database not in synch, job not committed'
                else:
                    self._dataBase.set_status(h, ACTV)
                l.append(h)
        finally:
            self._condV.release()
        if l:
            l += self._dataBase.get_transfer_info(h)
            if len(l) != 5:
                raise RuntimeError,\
                      'Incomplete/Incorrect transfer info for handle'
        return tuple(l)
    # ----------------------------------------------------------------
    #  get_record: throw RuntimeError if len of tuple is wrong.
    #     t = tuple('prototype', 'source url', 'destination url', \
    #               'marker', 'status', 'priority')
    # ----------------------------------------------------------------
    def get_record(self, handle):
        t = self._dataBase.get_record(handle)
        if len(t) != RECORD_LEN:
            raise RuntimeError, 'Incomplete/Incorrect transfer info for handle'
        return t
    # ----------------------------------------------------------------
    #  change_priority: throws ValueError if priority of handle
    #     cannot be changed, else the handle is at new priority.
    #     -PRE:   handle status must be in CHANGE_PRIOR_STATE.
    #             QUEUE_STATE contains queued states.
    # ----------------------------------------------------------------
    def change_priority(self, handle, new_priority):
        self.__check_priority(new_priority)
        self._condV.acquire(1)
        try:
            s = self._dataBase.get_status(handle)
            for state in CHANGE_PRIOR_STATE:
                if s == state:
                    if s == COMT:
                        p = self._dataBase.get_priority(handle)
                        if p != new_priority:
                            self._queueM.move(handle, new_priority)
                            self._dataBase.set_priority(handle, new_priority)
                    else:
                        self._dataBase.set_priority(handle, new_priority)
                    break
            else:
                raise StatusError('job not in a changeable priority status',
                                  g_status_dict.get(s))
        finally:
            self._condV.release()
        return handle
    # ----------------------------------------------------------------
    #  clear_record: not in queue, clear inactive job's database record.
    #    throws ValueError if job in activated state.
    # ----------------------------------------------------------------
    def clear_record(self, handle):
        self._condV.acquire(1)
        try:
            s = self._dataBase.get_status(handle)
            for aState in ACTIVE_STATES:
                if s == aState:
                    raise StatusError('job in activated status',
                                      g_status_dict.get(s))
            else:
                self._dataBase.clear_entry(handle)
        finally:
            self._condV.release()
        return
    # ----------------------------------------------------------------
    #  ACTV -> ADM_STOP, USER_STOP, DONE, FAIL
    #  COMT -> FAIL
    #  stop_active: stop an active state job, database status is
    #     adm_stop/user_stop.  throws ValueError
    # ----------------------------------------------------------------
    def adm_stop_active(self, handle):
        self._condV.acquire(1)
        try:
            s = self._dataBase.get_status(handle)
            if s == ACTV:
                self._dataBase.set_status(handle, ADM_STOP)
            elif s == COMT:
                self._queueM.remove(handle)
                self._dataBase.set_status(handle, FAIL)
            else:
                raise StatusError('job not in an activated status',
                                  g_status_dict.get(s))
        finally:
            self._condV.release()
        return
    def user_stop_active(self, handle):
        self._condV.acquire(1)
        try:
            s = self._dataBase.get_status(handle)
            if s == ACTV:
                self._dataBase.set_status(handle, USER_STOP)
            elif s == COMT:
                self._queueM.remove(handle)
                self._dataBase.set_status(handle, FAIL)
            else:
                raise StatusError('job not in an activated status',
                                  g_status_dict.get(s))
        finally:
            self._condV.release()
        return
    def transfer_done(self, handle):
        self._condV.acquire(1)
        try:
            s = self._dataBase.get_status(handle)
            if s == ACTV:
                self._dataBase.set_status(handle, DONE)
            else:
                raise StatusError('Only an Active process can move to Done',
                                  g_status_dict.get(s))
        finally:
            self._condV.release()
        return
    def transfer_failed(self, handle):
        s = self._dataBase.get_status(handle)
        if s == ACTV:
            self._dataBase.set_status(handle, FAIL)
        else:
            raise StatusError('Only an Active process can move to Fail',
                              g_status_dict.get(s))
        return

