# ----------------------------------------------------------------
#  Util module:
#    Classes
#        StatusError: Status exception class
#        Logger: singleton log utility 
#   Functions:
#        Log_exception:
#        Log_error:
#        Log_event:
#        Log_debug:
#        throw:
# ----------------------------------------------------------------
import sys, exceptions, traceback, time, threading
try:
    import NetLogger
except ImportError, ex:
    print "ImportError %s" %(ex.args[0])
    print "Can't find NetLogger.py, Netlogging unavailable"
# ----------------------------------------------------------------
#  Constants
# ----------------------------------------------------------------
LO_PRIORITY = 0
HI_PRIORITY = 1
RECORD_LEN  = 5
# ----------------------------------------------------------------
# status constants/information
#
# ----------------------------------------------------------------
SUBM = 0
COMT = 1
ACTV = 2
DONE = 3
FAIL = 4
STOP = 5
USER_STOP = 6
ADM_STOP = 7
MAX_STATUS = 7
g_status_dict = {SUBM:'submit', COMT:'commit', ACTV:'active', DONE:'done', 
                  FAIL:'fail', STOP:'stop', USER_STOP:'user stop',
                 ADM_STOP:'adm stop'}
ACTIVE_STATES = (COMT, ACTV)
PT_STATE = (USER_STOP, ADM_STOP)
COMMIT_STATES = (SUBM, STOP, USER_STOP, ADM_STOP)
CHANGE_PRIOR_STATE = (COMT, SUBM)
QUEUE_STATE = (COMT,)

# ----------------------------------------------------------------
#  StatusError: Attempted Violation of allowable state transitions.
#                <-------------------\
#  SUBM --> COMT --> ACTV  --> PARTIAL_TRANS (ADM_STOP,USER_STOP,STOP)
#                \--> FAIL  \--> FAIL, DONE
#
# ----------------------------------------------------------------
class StatusError(exceptions.Exception):
    pass

# ----------------------------------------------------------------
# urlparser
# ----------------------------------------------------------------
class urlparser:
    def __init__(self):
        self._tok1 = '://'
        self._tok2 = '/'
        return
    def urlparse(self, url):
        begIdx = url.find(self._tok1)
        if begIdx <= 0:
            raise ValueError(('incorrect format for url(1) %s' %(url)))
        scheme = url[:begIdx]
        begIdx += len(self._tok1)
        endIdx = url[begIdx:].find(self._tok2)
        if endIdx <= 0:
            raise ValueError(('incorrect format for url(2) %s' %(url)))
        endIdx += begIdx
        location = url[begIdx:endIdx]
        path = url[endIdx:]
        return (scheme, location, path)
    def urlunparse(self, *fragments):
        tmp = fragments[0] + self._tok1 + fragments[1]
        for i in fragments[2:]:
            tmp += i
        return tmp

# ----------------------------------------------------------------
# Logger
#  debug_levels
#    0 -
#    1 -
#    2 -
#    3 -
#    4 - log exception tracebacks
#    5 - log threads 
# ----------------------------------------------------------------
def get_Logger():
    if not Logger.singleton:
        x = Logger()
    return Logger.singleton

class Logger:
    singleton = None
    def __init__(self, file=None, netlog_url=None, netlog_name=('black label'), debug=0):
        if Logger.singleton:
            throw(RuntimeError('singleton class, use get_Logger method'))
        Logger.singleton = self
        self._file = None
        self._netlog = None
        self._debug = debug
        if file:
            self._file = open(file, 'a')
            head = [('\n-- ' + time.ctime() + ('-' * 30) + '\n')]
            head.append((('-' * (len(head[0]) - 1) + '\n')))
            self._file.writelines(head)
            sys.stdout = self._file
            sys.stderr = self._file
            self._file.flush()
        if netlog_url:
            print netlog_url
            self._netlog = NetLogger.NetLogger(netlog_name)
            flags = 0
            #flags = flags | NetLogger.NL_APPEND
            #flags = flags | NetLogger.NL_MEM
            #flags = flags | NetLogger.NL_ENV
            try:
                err = self._netlog.nlOpen(netlog_url, flags)
            except Exception, ex:
                ex.args += (("error on NetLoggerOpen"),)
                raise
        return
    def __del__(self):
        if self._file:
            self._file.flush()
            self._file.close()
        if self._netlog:
            self._netlog.nlClose()
        del singleton
    def log_message(self, msg, debug=0):
        if debug <= self._debug:
            if not self._file: print msg
            else:
                self._file.write((msg + '\n'))
                self._file.flush()
        return
    def log_exception(self, excpt):
        if not self._file:
            for i in excpt.args: print '\t', i
        else:
            msg = str(excpt)
            for i in excpt.args: 
                try: 
                    msg += ('\t' + i +'\n')
                except TypeError, ex:
                    msg = str(excpt.args) + '\n'
                    break
            self._file.write(msg)
            self._file.flush()
        if self._debug > 3:
            ex_info = sys.exc_info()
            traceback.print_tb(ex_info[2], 100, self._file)
        return
    def log_threads(self, debug):
        if debug <= self._debug:
            msg = ''
            for i in threading.enumerate():
                msg += (str(i) + '\n')
            if not self._file: print msg
            else:
                self._file.write(msg)
                self._file.flush()
        return        
    def log_netlogger(self, event, msg):
        if self._netlog:
            self._netlog.nlWrite(event, msg)
            self._netlog.nlFlush()
def Log_exception(ex):
    logger = get_Logger()
    logger.log_exception(ex)
    return
def Log_error(msg):
    logger = get_Logger()
    logger.log_message(msg)
    return
def Log_event(msg):
    logger = get_Logger()
    logger.log_message(msg)
    return
def Log_debug(msg, debug):
    logger = get_Logger()
    logger.log_message(msg, debug)
    return
def Log_threads(debug):
    logger = get_Logger()
    logger.log_threads(debug)
    return
def Log_netlogger(event, msg):
    logger = get_Logger()
    logger.log_netlogger(event, msg)
def throw(ex):
    Log_exception(ex)
    raise ex
