# ------------------------------------------------------------------------------
# ----------------------------------------------------------------
#  rftClient.py: Reliable File Transfer client library routines
#     classes:
#         rftClient
#     functions:
#         setDefaults()
# ----------------------------------------------------------------
import sys, traceback, socket, getopt, ConfigParser, traceback
import pyGlobus
from pyGlobus import GSISOAP
from pyGlobus.GSISOAP import ioc, SOAPProxy
from Util import urlparser

requests = ('completeTransferRequest', 'submitTransferRequest' ,
            'commitTransferRequest', 'listTransferRequest' ,
            'checkStatusRequest' , 'changePriorityRequest',
            'clearRecordRequest' , 'userStopTransferRequest' ,
            'printDS' , 'getDump')
# ----------------------------------------------------------------
#  rftClient:
#
# ----------------------------------------------------------------
class rftClient:
     def __init__(self, scheme, hostname, port, namespace):
          up = urlparser()
          uri = up.urlunparse(scheme, hostname)
          self._server = SOAPProxy(('%s:%d' %(uri, port)), namespace)
          self._server.channel_mode = ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_SSL_WRAP
          self._server.delegation_mode = \
                                       ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_NONE
          return
     def completeTransferRequest(self, handle, src_url, dst_url, priority):
          up = urlparser()
          t_src = up.urlparse(src_url)
          t_dst = up.urlparse(dst_url)
          if t_src[0] != t_dst[0]:
               raise ValueError, 'transfer protocol inconistent between src and dst'
          src = (t_src[1] + t_src[2])
          dst = (t_dst[1] + t_dst[2])
          h = self._server.completeTransferRequest(handle, src_url, dst_url, priority)
          return h
     def submitTransferRequest(self, src_url, dst_url, priority):
          up = urlparser()
          t_src = up.urlparse(src_url)
          t_dst = up.urlparse(dst_url)
          if t_src[0] != t_dst[0]:
               raise ValueError, 'transfer protocol inconistent between src and dst'
          h = self._server.submitTransferRequest(src_url, dst_url, priority)
          return h
     def commitTransferRequest(self, handle):
          h = self._server.commitTransferRequest(handle)
          return h
     def listTransferRequest(self, handle):
          h = self._server.listTransferRequest(handle)
          return h
     def checkStatusRequest(self, handle):
          s = self._server.checkStatusRequest(handle)
          return s
     def changePriorityRequest(self, handle, priority):
          h = self._server.changePriorityRequest(handle, priority)
          return h
     def clearRecordRequest(self, handle):
          h = self._server.clearRecordRequest(handle)
          return h
     def userStopTransferRequest(self, handle):
          h = self._server.userStopTransferRequest(handle)
          return h
     def printDS(self):
          tmp = self._server.printDS()
          return tmp
     def getDump(self):
          try:
              dump = self._server.getDump()
          except Exception, ex:
              ex_info = sys.exc_info()
              traceback.print_tb(ex_info[2], 100)
          return dump

# ----------------------------------------------------------------
#  setDefaults: read default values for [Client] from
#      default.cfg
# ----------------------------------------------------------------
def setDefaults():
     cp = ConfigParser.ConfigParser()
     cp.readfp(open('defaults.cfg'))
     cp_section = 'Client'
     port            = cp.getint(cp_section, 'port')
     namespace       = cp.get(cp_section, 'namespace')
     proto           = (cp.get(cp_section, 'trans_protocol'))
     soap_protocol   = cp.get(cp_section, 'soap_protocol')
     debug           = cp.getint(cp_section, 'debug_level')
     del cp
     del cp_section
     return (port, namespace, proto, soap_protocol, debug)
