#!/usr/bin/env python
# ----------------------------------------------------------------
#  setup.py
# ----------------------------------------------------------------
__version__ = "$Revision: 1.4 $"
import sys, os, getopt
from distutils import sysconfig
from distutils import text_file
from distutils.errors import *
from distutils.core import Extension, setup
from distutils.command.build_ext import build_ext

#----------------------------------------------------------------------
# User Configurable 
#----------------------------------------------------------------------
SOAP_MODULE = 'GSISOAP'

#----------------------------------------------------------------------
#  Description
#----------------------------------------------------------------------
PKGDIR = './'
VERSION = '0.1'
DESCRIPTION = "reliable file transfer manager",
LONG_DESCRIPTION = ''
AUTHOR = 'Joshua Boverhof'
AUTHOR_EMAIL = 'JRBoverhof@lbl.gov'
URL = 'http://www-itg.lbl.gov/rftm/'
LICENCE = ''
SOAP_DISTRIBUTIONS = ['GSISOAP', 'SOAP']

#----------------------------------------------------------------------
# Do the Setup/Build/Install
#----------------------------------------------------------------------

"""
  WEB100 = "/home/g5/users/kjackson/web100-src/web100/util/src"
  EXTRAOBJS = "%s%s" % (WEB100, "/web100lib.o")

  setup(name="Distutils", version="1.0", description="Python Web100 wrapper",
      author="Keith Jackson", author_email="KRJackson@lbl.gov",
      url="https://internal.web100.org/project/showfiles.php?group_id=3",
      py_modules=['PyWeb100'],
      ext_modules=[Extension("web100", ["web100.i"], include_dirs=[WEB100],
                  library_dirs=[WEB100], extra_objects=[EXTRAOBJS])])
"""




def main():
    if SOAP_MODULE == 'GSISOAP':
        setup(name             = PKGDIR,
              version          = VERSION,
              description      = DESCRIPTION,
              long_description = LONG_DESCRIPTION,
              author           = AUTHOR,
              author_email     = AUTHOR_EMAIL,
              url              = URL,
              licence          = LICENCE,
              py_modules=['DataManager', 'FileManager', 'Util'],
              ext_package = PKGDIR,
              data_files=['gadfly.py']
              )

def find_file(filename, std_dirs, paths):
    """Searches for the directory where a given file is located,
    and returns a possibly-empty list of additional directories, or None
    if the file couldn't be found at all.

    'filename' is the name of a file, such as readline.h or libcrypto.a.
    'std_dirs' is the list of standard system directories; if the
        file is found in one of them, no additional directives are needed.
    'paths' is a list of additional locations to check; if the file is
        found in one of them, the resulting list will contain the directory.
    """

    # Check the standard locations
    for dir in std_dirs:
        f = os.path.join(dir, filename)
        if os.path.exists(f): return []

    # Check the additional directories
    for dir in paths:
        f = os.path.join(dir, filename)
        if os.path.exists(f):
            return [dir]

    # Not found anywhere
    return None

def find_library_file(compiler, libname, std_dirs, paths):
    filename = compiler.library_filename(libname, lib_type='shared')
    result = find_file(filename, std_dirs, paths)
    if result is not None: return result

    filename = compiler.library_filename(libname, lib_type='static')
    result = find_file(filename, std_dirs, paths)
    return result

def module_enabled(extlist, modname):
    """Returns whether the module 'modname' is present in the list
    of extensions 'extlist'."""
    extlist = [ext for ext in extlist if ext.name == modname]
    return len(extlist)

if __name__ == "__main__":
    main()
